# Builtin modules
from __future__ import annotations
import sys, atexit, traceback, unittest
from datetime import datetime
from time import monotonic
from threading import RLock
from typing import Dict, List, Any, cast, TextIO, Union, Iterable, Tuple, Optional, Callable
# Local modules
# Program
DEF_FILTER:List[Dict[str, Any]] = [{ "*":"TRACE" }]
DEF_FORMAT:str = "[{levelshortname}][{date}][{name}] : {message}\n"
DEF_DATE:str = "%Y-%m-%d %H:%M:%S.%f"

class LoggerManager:
	lock:RLock = RLock()
	handler:Optional[LoggerManager] = None
	filterChangeTime:float = monotonic()
	groupSeperator:str = "."
	def __init__(self, filter:Union[list, str, Filter]=DEF_FILTER, messageFormat:str=DEF_FORMAT, dateFormat:str=DEF_DATE,
	defaultLevel:Union[int, str]="WARNING", hookSTDOut:bool=True, hookSTDErr:bool=True):
		if isinstance(self.handler, LoggerManager):
			raise RuntimeError("LoggerManager already initialized")
		LoggerManager.handler = self
		self.delta:float = monotonic()
		self.messageFormat:str = messageFormat
		self.dateFormat:str = dateFormat
		self.modules:List[ModuleBase] = []
		self.filter:Filter = Filter(Levels.parse(defaultLevel))
		self._extendFilter(filter)
		self._stderr:TextIO = sys.stderr
		self._stdout:TextIO = sys.stdout
		self._excepthook:Any = sys.excepthook
		if hookSTDErr:
			sys.stderr = cast(TextIO, STDErrModule())
			sys.excepthook = lambda *a: sys.stderr.write("".join(traceback.format_exception(*a)))
		if hookSTDOut:
			sys.stdout = cast(TextIO, STDOutModule())
		atexit.register(self.close)
	@staticmethod
	def getFilterData(name:str) -> Tuple[float, int]:
		if isinstance(LoggerManager.handler, LoggerManager):
			return LoggerManager.handler._getFilterData(name)
		return LoggerManager.filterChangeTime, 0
	def _getFilterData(self, name:str) -> Tuple[float, int]:
		return (
			self.filterChangeTime,
			self.filter.getFilteredID(
				name.split(self.groupSeperator)
			)
		)
	@staticmethod
	def emit(name:str, levelID:int, timestamp:float, message:Any, _args:Tuple[Any, ...], _kwargs:Dict[str, Any]) -> None:
		if isinstance(LoggerManager.handler, LoggerManager):
			LoggerManager.handler._emit(name, levelID, timestamp, message, _args, _kwargs)
	def _emit(self, name:str, levelID:int, timestamp:float, message:Any, _args:Tuple[Any, ...], _kwargs:Dict[str, Any]) -> None:
		parsedMessage:str = self.messageFormatter(name, levelID, timestamp, message, _args, _kwargs)
		with self.lock:
			for handler in self.modules:
				try: handler.emit(parsedMessage)
				except: pass
	@staticmethod
	def extendFilter(data:Union[list, str, Filter]) -> None:
		if isinstance(LoggerManager.handler, LoggerManager):
			LoggerManager.handler._extendFilter(data)
	def _extendFilter(self, data:Union[list, str, Filter]) -> None:
		filter:Filter = Filter(0)
		if isinstance(data, list):
			filter = FilterParser.fromJson(data)
		elif isinstance(data, str):
			filter = FilterParser.fromString(data)
		assert isinstance(filter, Filter)
		self.filter.extend(filter)
	@staticmethod
	def close() -> None:
		if isinstance(LoggerManager.handler, LoggerManager):
			LoggerManager.handler._close()
	def _close(self) -> None:
		module:ModuleBase
		for module in self.modules:
			try:
				module.close()
			except:
				pass
		self.modules.clear()
		if isinstance(sys.stderr, STDErrModule):
			sys.stderr.forceFlush()
			sys.stderr = self._stderr
		if isinstance(sys.stdout, STDOutModule):
			sys.stdout.forceFlush()
			sys.stdout = self._stdout
		LoggerManager.handler = None
	@staticmethod
	def getLogger(self, name:str) -> "Logger":
		return Logger(name)
	def messageFormatter(self, name:str, levelID:int, timestamp:float, message:str,
	_args:Tuple[Any, ...], _kwargs:Dict[str, Any], datetime:Any=datetime) -> str:
		args:Tuple[Any, ...] = tuple(map(lambda v: v() if callable(v) else v, _args))
		kwargs:Dict[str, Any] = dict(map(lambda d: (d[0], (d[1]() if callable(d[1]) else d[1])), _kwargs.items()))
		return self.messageFormat.format(
			levelnumber=levelID,
			levelname=Levels.getLevelNameByID(levelID),
			levelshortname=Levels.getLevelShortNameByID(levelID),
			date=datetime.utcfromtimestamp(timestamp).strftime(self.dateFormat),
			timestamp=timestamp,
			ellapsed=timestamp - self.delta,
			message=message.format(*args, **kwargs) if args or kwargs else message,
			name=name
		)
	def initStandardOutStream(self) -> None:
		self.modules.append( STDOutStreamingModule(self._stdout) )
	def initFileStream(self, fullPath:str) -> None:
		self.modules.append( FileStream(fullPath) )
	def initRotatedFileStream(self, fullPath:str, maxBytes:int=0, rotateDaily:bool=False, maxBackup:Optional[int]=None) -> None:
		self.modules.append( RotatedFileStream(fullPath, maxBytes, rotateDaily, maxBackup) )
	def initDailyFileStream(self, logPath:str, prefix:str, postfix:str, dateFormat:str="%Y-%m-%d") -> None:
		self.modules.append( DailyFileStream(logPath, prefix, postfix, dateFormat) )

class DowngradedLoggerManager(LoggerManager):
	import logging
	def __init__(self):
		LoggerManager.handler = self
		self.logging = logging
	def _emit(self, name:str, levelID:int, timestamp:float, message:Any, _args:Tuple[Any, ...], _kwargs:Dict[str, Any]) -> None:
		args:Tuple[Any, ...] = tuple(map(lambda v: v() if callable(v) else v, _args))
		kwargs:Dict[str, Any] = dict(map(lambda d: (d[0], (d[1]() if callable(d[1]) else d[1])), _kwargs.items()))
		self.logging.getLogger(name).log(
			self.logging._nameToLevel.get({
				"CRITICAL":"CRITICAL",
				"ERROR":"ERROR",
				"WARNING":"WARNING",
				"INFO":"INFO",
				"DEBUG":"DEBUG",
				"TRACE":"DEBUG",
			}.get(Levels.getLevelNameByID(levelID), "NOTSET"), 0),
			message.format(*args, **kwargs) if args or kwargs else message
		)
	def _getFilterData(self, name:str) -> Tuple[float, int]:
		return time(), 0

class LoggerManagerTest(unittest.TestCase):
	def test_first(self):
		from tempfile import TemporaryDirectory
		lm = LoggerManager(
			messageFormat="[{levelshortname}][{name}] : {message}\n",
			defaultLevel="TRACE",
			hookSTDOut=False,
			hookSTDErr=False
		)
		log = Logger("test")
		lm.initStandardOutStream()
		log.info("If you see this i'm working well")
		with TemporaryDirectory() as tmpdir:
			fn:str = "{}/teszt.log".format(tmpdir)
			lm.initFileStream(fn)
			log.info("Hello")
			with open(fn, "rt") as fid:
				self.assertEqual(fid.read(), "[INF][test] : Hello\n")
		lm.close()
	def test_second(self):
		from tempfile import TemporaryDirectory
		lm = LoggerManager(
			messageFormat="[{levelshortname}][{name}] : {message}\n",
			defaultLevel="TRACE",
			hookSTDOut=True,
			hookSTDErr=False
		)
		with TemporaryDirectory() as tmpdir:
			fn:str = "{}/teszt.log".format(tmpdir)
			lm.initFileStream(fn)
			print("Hel", end="")
			print("lo")
			print("Hello")
			with open(fn, "rt") as fid:
				self.assertEqual(fid.read(), "[INF][Standard.Output] : Hello\n[INF][Standard.Output] : Hello\n")
		lm.close()

# Finalizing imports
from .modules import *
from .levels import Levels
from .filters import Filter, FilterParser
from .logger import Logger



# Builtin modules
from __future__ import annotations
import unittest
from typing import List, Union, Tuple
# Local modules
# Program
class Levels:
	levels:List[ Tuple[int, str, str] ] = [
		(100, "DISABLED", "DIS"),
		(60, "CRITICAL", "CRI"),
		(50, "ERROR", "ERR"),
		(40, "WARNING", "WAR"),
		(30, "INFO", "INF"),
		(20, "DEBUG", "DBG"),
		(10, "TRACE", "TRC"),
	]
	@classmethod
	def addLevel(self, id:int, name:str, shortName:str):
		assert id > 0, "ID must be higher as zero"
		self.levels.append((id, name, shortName))
		self.levels = sorted(self.levels, key=lambda x: -x[0])
	@classmethod
	def getLevelIDByName(self, name:str) -> int:
		name = name.upper()
		d:Tuple[int, str, str]
		for d in self.levels:
			if d[1] == name:
				return d[0]
		return 0
	@classmethod
	def getLevelIDByShortName(self, shortName:str) -> int:
		shortName = shortName.upper()
		d:Tuple[int, str, str]
		for d in self.levels:
			if d[2] == shortName:
				return d[0]
		return 0
	@classmethod
	def getLevelNameByID(self, id:int) -> str:
		d:Tuple[int, str, str]
		for d in self.levels:
			if d[0] == id:
				return d[1]
		raise KeyError("Levels: Unknown level: {}".format(id))
	@classmethod
	def getLevelShortNameByID(self, id:int) -> str:
		d:Tuple[int, str, str]
		for d in self.levels:
			if d[0] == id:
				return d[2]
		raise KeyError("Levels: Unknown level: {}".format(id))
	@classmethod
	def parse(self, level:Union[int, str]) -> int:
		r:int
		if isinstance(level, str) and level.isdigit():
			level = int(level)
		if isinstance(level, int):
			for d in self.levels:
				if d[0] == level:
					return level
		else:
			r = self.getLevelIDByName(level)
			if r == 0:
				r = self.getLevelIDByShortName(level)
			if r != 0:
				return r
		raise KeyError("Levels: Unknown level: {}".format(level))

class LevelsTest(unittest.TestCase):
	def test_parser(self):
		self.assertEqual( Levels.parse("DISAbLED"), 100 )
		self.assertEqual( Levels.parse("WARnING"), 40 )
		self.assertEqual( Levels.parse("wAr"), 40 )
		with self.assertRaises(KeyError):
			Levels.parse("NOTHING")
		with self.assertRaises(KeyError):
			Levels.parse(5)
		Levels.addLevel(5, "LOWLEVEL", "LOW")
		self.assertEqual( Levels.parse("LOWLEVEL"), 5 )
		self.assertEqual( Levels.parse("LOW"), 5 )
		self.assertEqual( Levels.parse(5), 5 )
	def test_gets(self):
		self.assertEqual( Levels.getLevelNameByID(40), "WARNING" )
		self.assertEqual( Levels.getLevelShortNameByID(40), "WAR" )
		with self.assertRaises(KeyError):
			Levels.getLevelNameByID(15)
		with self.assertRaises(KeyError):
			Levels.getLevelShortNameByID(15)



# Builtin modules
from __future__ import annotations
import json, re, unittest
from fnmatch import fnmatchcase
from typing import Dict, List, Any, Union, Tuple, Optional, cast
from collections import OrderedDict
# Local modules
from . import Levels
# Program
class Filter:
	__slots__ = "keys", "fallbackLevel",
	def __init__(self, fallbackLevel:int):
		self.keys:Dict[str, Filter] = cast(Dict[str, Filter], OrderedDict())
		self.fallbackLevel = fallbackLevel
	def addLogger(self, k:str, v:Filter) -> Filter:
		self.keys[k] = v
		return self
	def setFallbackLevel(self, level:Union[int, str]):
		self.fallbackLevel = Levels.parse(level)
	def getKey(self, k:str) -> Optional[Filter]:
		if k.lower() in self.keys:
			return self.keys[k.lower()]
		return None
	def getFilteredID(self, path:List[str]) -> int:
		name:str = path.pop(0)
		key:str
		val:Filter
		for key, val in reversed(self.keys.items()): # type: ignore
			if name == key or fnmatchcase(name, key):
				if path:
					return val.getFilteredID(path) or self.fallbackLevel
				else:
					return val.fallbackLevel or self.fallbackLevel
		return self.fallbackLevel
	def dump(self) -> list:
		ret:list = [{ "*":self.fallbackLevel }]
		key:str
		val:Filter
		for key, val in self.keys.items():
			ret.append({ key:val.dump() })
		return ret
	def extend(self, inp:Filter) -> None:
		key:str
		val:Union[int, Filter]
		if inp.fallbackLevel != 0:
			self.fallbackLevel = inp.fallbackLevel
		for key, val in inp.keys.items():
			if key == "*":
				self.fallbackLevel = cast(int, val)
			else:
				if key not in self.keys:
					self.keys[key] = Filter(0)
				self.keys[key].extend(cast(Filter, val))

class FilterParser:
	@classmethod
	def fromString(self, data:str) -> Filter:
		"""
		parent:ERROR,parent.children.son:WARNING
		->
		[
			{ "*": 0 },
			{ "parent": [
				{ "*": 50 },
				{ "children": [
					{ "*": 0 },
					{ "son": [
						{ "*": 40 }
					]}
				]}
			]}
		]
		"""
		paths:List[str]
		rawPaths:str
		levelID:str
		lastScope:Filter
		ret:Filter = Filter(0)
		i:int
		for part in data.lower().split(","):
			rawPaths, levelID = part.split(":")
			paths = rawPaths.split(LoggerManager.groupSeperator)
			lastScope = ret
			for i, path in enumerate(paths):
				if path not in lastScope.keys:
					lastScope.keys[path] = Filter(Levels.parse(levelID) if i == len(paths)-1 else 0)
				lastScope = lastScope.keys[path]
		return ret
	@classmethod
	def fromJson(self, datas:list) -> Filter:
		"""
		[
			{ "parent": [
				{ "*": 50 },
				{ "children": [
					{ "son": [
						{ "*": 40 }
					]}
				]}
			]}
		]
		->
		[
			{ "*": 0 },
			{ "parent": [
				{ "*": 50 },
				{ "children": [
					{ "*": 0 },
					{ "son": [
						{ "*": 40 }
					]}
				]}
			]}
		]
		"""
		data:Dict[str, Any]
		ret:Filter = Filter(0)
		for data in datas:
			for key in data.keys():
				if isinstance(data[key], list):
					ret.keys[key] = self.fromJson(data[key])
				elif key == "*":
					ret.fallbackLevel = Levels.parse(data[key])
				else:
					# Fallback for lazy input
					ret.keys[key.lower()] = self.fromJson([ {"*": Levels.parse(data[key])} ])
		return ret

class FilterTest(unittest.TestCase):
	def test(self):
		_old:str = LoggerManager.groupSeperator
		LoggerManager.groupSeperator = "-"
		beforeFilterData:list = [
			{ "server": [
				{ "client": [
					{ "*": 50 },
					{ "192.168.*": [
						{ "*": 40 },
					]},
					{ "192.168.1.*": [
						{ "*": 40 },
					]},
					{ "192.168.2.*": [
						{ "*": 20 },
						{ "sql": [
							{ "*": 40 },
						]},
					]},
					{ "192.168.2.1": [
						{ "*": 10 }
					]}
				]}
			]}
		]
		afterFilterData:list = [
			{ "*": 0 },
			{ "server": [
				{ "*": 0 },
				{ "client": [
					{ "*": 50 },
					{ "192.168.*": [
						{ "*": 40 },
					]},
					{ "192.168.1.*": [
						{ "*": 40 },
					]},
					{ "192.168.2.*": [
						{ "*": 20 },
						{ "sql": [
							{ "*": 40 },
						]},
					]},
					{ "192.168.2.1": [
						{ "*": 10 }
					]}
				]}
			]}
		]
		filter:Filter = FilterParser.fromJson(beforeFilterData)
		self.assertEqual( filter.dump(), afterFilterData )
		self.assertEqual( filter.getFilteredID(["some"]), 0)
		self.assertEqual( filter.getFilteredID(["server"]), 0 )
		self.assertEqual( filter.getFilteredID(["server", "client"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "255.255.255.255"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "255.255.255.255", "sql"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.0.0"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.1.0"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.1.2"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.1.2", "sql"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0"]), 20 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0", "result"]), 20 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0", "sql"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0", "sql", "execute"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.1", "sql"]), 10 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.1", "sql", "execute"]), 10 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.3", "sql", "execute"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.3", "somewhat"]), 20 )
		filter.extend( FilterParser.fromString("server:50,server-client-192.168.2.*:50,server-client-192.168.2.4-sql:50") )
		self.assertEqual( filter.getFilteredID(["server"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "255.255.255.255"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "255.255.255.255", "sql"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.0.0"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.1.0"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.1.2"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.1.2", "sql"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0", "result"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0", "sql"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.0", "sql", "execute"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.1", "sql"]), 10 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.1", "sql", "execute"]), 10 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.3", "sql", "execute"]), 40 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.3", "somewhat"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.4"]), 50 )
		self.assertEqual( filter.getFilteredID(["server", "client", "192.168.2.4", "somewhat"]), 50 )
		LoggerManager.groupSeperator = _old

from .loggerManager import LoggerManager



# Builtin modules
import logging, unittest
# Local modules
from .filters import FilterTest
from .levels import LevelsTest
from .loggerManager import LoggerManagerTest
# Program
logging.basicConfig(format='[%(levelname).3s][%(asctime)s][%(name)s]: %(message)s', level=logging.INFO)
unittest.main(verbosity=2)



# Builtin modules
import re, os, traceback
from abc import ABCMeta, abstractmethod
from glob import glob
from datetime import datetime
from functools import cmp_to_key
from time import time
from typing import List, Any, TextIO, Optional
# Local modules
# Program
class STDErrModule:
	def __init__(self) -> None:
		self.log:Any =Logger("Standard").getChild("Error")
		self.closed:bool = False
		self.buffer:str = ""
	def write(self, data:str) -> None:
		if data:
			self.buffer += data
		self.flush()
	def flush(self) -> None:
		while "\n" in self.buffer:
			pos:int = self.buffer.find("\n")
			self.log.error(self.buffer[:pos])
			self.buffer = self.buffer[pos+1:]
	def forceFlush(self) -> None:
		self.log.error(self.buffer)
		self.buffer = ""
	def close(self) -> None:
		self.closed = True

class STDOutModule:
	def __init__(self) -> None:
		self.log:Any = Logger("Standard").getChild("Output")
		self.closed:bool = False
		self.buffer:str = ""
	def write(self, data:str) -> None:
		if data:
			self.buffer += data
		self.flush()
	def flush(self) -> None:
		while "\n" in self.buffer:
			pos:int = self.buffer.find("\n")
			self.log.info(self.buffer[:pos])
			self.buffer = self.buffer[pos+1:]
	def forceFlush(self) -> None:
		self.log.info(self.buffer)
		self.buffer = ""
	def close(self) -> None:
		self.closed = True

class ModuleBase(metaclass=ABCMeta):
	@abstractmethod
	def emit(self, data:str) -> None: pass
	@abstractmethod
	def close(self) -> None: pass

class STDOutStreamingModule(ModuleBase):
	def __init__(self, stream:Any):
		self.stream:Any = stream
	def emit(self, data:str) -> None:
		if self.stream:
			try:
				self.stream.write(data)
				self.stream.flush()
			except:
				pass
	def close(self) -> None:
		if self.stream:
			self.stream.close()
		self.stream = None

class FileStream(ModuleBase):
	def __init__(self, fullPath:str):
		self.fullPath:str = fullPath
		self.stream:Any = None
		self.open()
	def open(self) -> None:
		try:
			os.makedirs( os.path.dirname(self.fullPath), 0o755 , True)
		except:
			traceback.print_exc()
		try:
			self.stream = open(self.fullPath, "at")
		except:
			traceback.print_exc()
	def write(self, data:str) -> None:
		if self.stream is not None:
			try:
				self.stream.write(data)
				self.stream.flush()
			except:
				traceback.print_exc()
	def emit(self, message:str) -> None:
		self.write(message)
	def close(self) -> None:
		if self.stream is not None:
			self.stream.close()
		self.stream = None

class RotatedFileStream(FileStream):
	def __init__(self, fullPath:str, maxBytes:int=0, rotateDaily:bool=False, maxBackup:Optional[int]=None):
		super().__init__(fullPath)
		self.maxBytes:int = maxBytes
		self.rotateDaily:bool = rotateDaily
		self.maxBackup:Optional[int] = maxBackup
		self.lastRotate:Optional[str] = None
		self.lastFileSize:Optional[int] = None
	def emit(self, message:str) -> None:
		if self.stream is not None:
			if self.shouldRotate(message):
				self.doRotate()
			super().emit(message)
	def shouldRotate(self, message:str) -> bool:
		if self.lastRotate is None:
			self.lastRotate = datetime.utcnow().strftime("%D")
			return True
		if self.maxBytes > 0:
			if self.lastFileSize is None:
				self.stream.seek(0, 2)
				self.lastFileSize = self.stream.tell()
			self.lastFileSize += len(message)
			if self.lastFileSize >= self.maxBytes:
				return True
		if self.rotateDaily:
			if self.lastRotate != datetime.utcnow().strftime("%D"):
				self.lastRotate = datetime.utcnow().strftime("%D")
				return True
		return False
	def doRotate(self) -> None:
		if self.stream is not None:
			self.stream.close()
			self.stream = None
		try:
			self.shiftLogFiles()
		except:
			traceback.print_exc()
		self.open()
	def shiftLogFiles(self) -> None:
		def sortFileNums(a:str, b:str) -> int:
			def parseFileNum(e:str) -> int:
				if not e:
					return 0
				r = re.findall(r'^.*[^\.]\.([0-9]*)$', e)
				if r:
					return int(r[0])
				else:
					return 0
			q:int = parseFileNum(a)
			w:int = parseFileNum(b)
			if q > w:
				return -1
			else:
				return 1
		if len(glob("%s" % self.fullPath)) == 0:
			return
		files:List[str] = [self.fullPath] + glob("{}.[0-9]*".format(self.fullPath))
		files.sort(key=cmp_to_key(sortFileNums))
		tmpFiles:List[str] = []
		file:str
		for file in files:
			if os.stat(file).st_size > 0:
				os.rename(file, "{}_tmp".format(file))
				tmpFiles.append("{}_tmp".format(file))
			else:
				os.remove(file)
		i:int
		for i, file in list(enumerate(tmpFiles[::-1])):
			if self.maxBackup is not None and ( self.maxBackup == 0 or self.maxBackup < i ):
				os.remove(file)
			else:
				os.rename(file, "{}.{}".format( self.fullPath, str(i+1).zfill(3) ))

class DailyFileStream(FileStream):
	def __init__(self, logPath:str, prefix:str="", postfix:str="", dateFormat:str="%Y-%m-%d"):
		self.path:str = logPath
		self.prefix:str = prefix
		self.postfix:str = postfix
		self.dateFormat:str = dateFormat
		self.lastRotate:Optional[str] = None
		super().__init__(self.buildPath())
	def buildPath(self) -> str:
		return "{}/{}{}{}".format(
			self.path,
			self.prefix,
			datetime.utcnow().strftime(self.dateFormat),
			self.postfix,
		)
	def emit(self, message:str) -> None:
		if self.stream is not None:
			if self.shouldRotate(message):
				self.doRotate()
			super().emit(message)
	def shouldRotate(self, message:str) -> bool:
		if self.lastRotate is None or self.lastRotate != datetime.utcnow().strftime("%D"):
			self.lastRotate = datetime.utcnow().strftime("%D")
			return True
		return False
	def doRotate(self) -> None:
		if self.stream is not None:
			self.stream.close()
			self.stream = None
		self.fullPath = self.buildPath()
		self.open()

# Finalizing imports
from .logger import Logger



#!/usr/bin/env python3
import os, shutil
import fsLogger
try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup

pwd = os.path.abspath(os.path.dirname(__file__))

setup(
	name = "python-fslogger",
	version = fsLogger.__version__,
	description = "Logging utility",
	keywords = "logging utility fusion solutions fusionsolutions",
	author = "Andor `iFA88` Rajci - Fusions Solutions KFT",
	author_email = "ifa@fusionsolutions.io",
	url = "https://github.com/FusionSolutions/python-fslogger",
	license = "GPL-3",
	package_dir={"fsLogger": "fsLogger"},
	packages=["fsLogger"],
	long_description=open(os.path.join(pwd, "README.md")).read(),
	long_description_content_type="text/markdown",
	zip_safe=False,
	python_requires=">=3.7.0",
	classifiers=[ # https://pypi.org/pypi?%3Aaction=list_classifiers
		"Development Status :: 4 - Beta",
		"Topic :: Utilities",
		"Programming Language :: Python :: 3 :: Only",
		"Programming Language :: Python :: 3.7",
		"Programming Language :: Python :: 3.8",
		"Programming Language :: Python :: 3.9",
		"License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
	],
)


# Builtin modules
from __future__ import annotations
import re
from typing import Dict, Tuple, Any, Union, Callable, Optional
from functools import partial
from time import time
# Local modules
# Program
class Logger:
	name:str
	filterChangeTime:float
	lastFilterLevel:int
	trace:Optional[Callable]
	debug:Optional[Callable]
	info:Optional[Callable]
	warn:Optional[Callable]
	warning:Optional[Callable]
	error:Optional[Callable]
	critical:Optional[Callable]
	fatal:Optional[Callable]
	__slots__ = (
		"name", "filterChangeTime", "lastFilterLevel", "trace", "debug",
		"info", "warn", "warning", "error", "critical", "fatal"
	)
	def __init__(self, name:Union[str, Logger]):
		if isinstance(name, Logger):
			name = name.name
		self.__setstate__({ "name":name, "filterChangeTime":0, "lastFilterLevel":0 })
	def __getstate__(self) -> Dict[str, Any]:
		return {
			"name":self.name,
			"filterChangeTime":self.filterChangeTime,
			"lastFilterLevel":self.lastFilterLevel,
		}
	def __setstate__(self, states:Dict[str, Any]) -> None:
		self.name = states["name"]
		self.filterChangeTime = states["filterChangeTime"]
		self.lastFilterLevel = states["lastFilterLevel"]
		self.trace    = partial(self._emit, Levels.getLevelIDByName("TRACE"))
		self.debug    = partial(self._emit, Levels.getLevelIDByName("DEBUG"))
		self.info     = partial(self._emit, Levels.getLevelIDByName("INFO"))
		self.warn     = partial(self._emit, Levels.getLevelIDByName("WARNING"))
		self.warning  = self.warn
		self.error    = partial(self._emit, Levels.getLevelIDByName("ERROR"))
		self.critical = partial(self._emit, Levels.getLevelIDByName("CRITICAL"))
		self.fatal    = self.critical
	def getChild(self, name:str) -> Logger:
		return Logger("{}{}{}".format(self.name, LoggerManager.groupSeperator, name))
	def isFiltered(self, levelID:Union[int, str]) -> bool:
		if self.filterChangeTime != LoggerManager.filterChangeTime:
			self.filterChangeTime, self.lastFilterLevel = LoggerManager.getFilterData(self.name)
		if isinstance(levelID, str):
			levelID = Levels.parse(levelID)
		return levelID >= self.lastFilterLevel
	def _emit(self, levelID:int, message:str, *args, **kwargs) -> None:
		if self.isFiltered(levelID):
			LoggerManager.emit(self.name, levelID, time(), message, args, kwargs)

# Finalizing imports
from .levels import Levels
from .loggerManager import LoggerManager



__version__ = "0.0.3"
__doc__ = """
Logging utility v{}
Copyright (C) 2021 Fusion Solutions KFT <contact@fusionsolutions.io>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/lgpl.txt>.
""".format(__version__)
from typing import Union, Optional
from .levels import Levels
from .filters import Filter, FilterParser
from .logger import Logger
from .loggerManager import LoggerManager, DowngradedLoggerManager

root:Optional[LoggerManager] = None
def SimpleLogger(level:Union[str, int]="TRACE") -> LoggerManager:
	global root
	if not isinstance(root, LoggerManager):
		root = LoggerManager(defaultLevel=level)
		root.initStandardOutStream()
	return root

def downgradeLoggerManager() -> LoggerManager:
	global root
	if not isinstance(root, LoggerManager):
		root = DowngradedLoggerManager()
	return root

__all__ = "LoggerManager", "Logger", "Levels", "FilterParser", "SimpleLogger", "downgradeLoggerManager"
